$(function() {
  var $sidebar = $('#sidebar:visible');

  /* utility */
  function getParameterByName(name, url) {
    if (!url) url = window.location.href;
    name = name.replace(/[\[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
  }

  /* create tile from map feature */
  var tile_popup = $('<div class="cell" id="tile-popup"></div>');
  tile_popup.css('bottom', $('nav').height() + 'px');
  tile_popup.appendTo($('#content'));
  tile_popup.on('click', function() {
    var $tile = $(this).children().first();
    $tile[0].matching_marker.matching_tile = null;
    $(this).empty();
  });
  $('div.cell.map').addClass('loading').on('combo:map-markers-ready', function() {
    $(this).removeClass('loading');
  });
  $('div.cell.map').on('combo:map-feature-prepare', function(ev, info) {
    info.layer.on('click', function(ev) {
      $('.leaflet-marker-icon.loading').removeClass('loading');
      if ($sidebar.length && info.layer.matching_tile) {
        $('#sidebar .cell').removeClass('selected');
        info.layer.matching_tile.addClass('selected');
        var $selected_cell = $('#sidebar .cell.selected');
        if ($selected_cell.prev().length) {
          $selected_cell.prependTo($selected_cell.parent());
        }
        return;
      }
      if ($sidebar.length == 0 && info.layer.matching_tile) {
        /* second click on same marker, hide the tile */
        $(tile_popup).empty();
        info.layer.matching_tile = null;
        return;
      }
      $(info.layer._icon).addClass('loading');
      if ($sidebar.length == 0) {
        /* click on marker without sidebar, remove current tile */
        var $tile = $(tile_popup).children().first();
        if ($tile.length) {
          $tile[0].matching_marker.matching_tile = null;
          tile_popup.empty();
        }
      }
      $.ajax({
         url: '/api/dashboard/auto-tile/' + info.layer.feature.properties.layer.identifier + '/',
         data: JSON.stringify(ev.target.feature.properties),
         contentType: 'application/json; charset=utf-8',
         type: 'POST',
         dataType: 'html',
         success: function(html) {
           $(info.layer._icon).removeClass('loading');
           var $tile = null;
           if ($sidebar.length == 0) {
             $tile = $(html);
             $tile.appendTo(tile_popup);
           } else {
             $('#sidebar .cell').removeClass('selected');
             var $tile = $('<div class="cell selected">' + html + '</div>');
             $tile.prependTo($sidebar);
             if ($tile.siblings().length > 1) {
               $tile.siblings().last()[0].matching_marker.matching_tile = null;
               $tile.siblings().last().remove();
             }
           }
           $tile[0].matching_marker = info.layer;
           info.layer.matching_tile = $tile;
         }
      });
    });
  });

  /* add & remove tile */
  $('div.cell, div#sidebar').delegate('a.remove-from-dashboard', 'click', function(e) {
    var dashboard = $(this).parents('.dashboardcell');
    var cell = $(this).parents('.cell')[0];
    var link = $(this);
    $.ajax({
      url: $(this).attr('href'),
      success: function(data) {
        if (dashboard.length) {
          /* if the user is on the dashboard, remove tile from screen */
          $(cell).hide(300);
        } else {
          $(link).hide();
          var $add_to_dashboard = $('<a class="add-to-dashboard"></a>');
          $add_to_dashboard.attr('href', data.cell_data.add_url);
          $add_to_dashboard.appendTo($(link).parent());
          $(link).remove();
        }
      }
    });
    e.stopPropagation();
    return false;
  });
  $('div.cell, div#sidebar').delegate('a.add-to-dashboard', 'click', function(e) {
    var link = $(this);
    $.ajax({
      url: $(this).attr('href'),
      success: function(data) {
        $(link).hide();
        var $remove_from_dashboard = $('<a class="remove-from-dashboard"></a>');
        $remove_from_dashboard.attr('href', data.cell_data.remove_url);
        $remove_from_dashboard.appendTo($(link).parent());
        $(link).remove();
        /* "pulse" dashboard menu entry */
        $('#nav').find('.menu-index').removeClass('pulse').addClass('pulse');
      }
    });
    e.stopPropagation();
    return false;
  });

  /* map search */
  var $map_widget = $('div.combo-cell-map');
  var $map_search_input = $('#mapsearch input');
  if ($map_widget.length) {
    var map = $map_widget[0].leaflet_map;
    var search_timeout_id = null;
    var base_geojson_url = $map_widget.data('geojson-url');
    $map_search_input.on('change keyup paste', function() {
      clearTimeout(search_timeout_id);
      search_timeout_id = setTimeout(function() {
        var query =  $('#mapsearch input').val();
        query = query.toLowerCase();
        query = query.replace(/^rue\s/i, '');
        if (query.length < 2) return;
        $map_widget.data('geojson-url', base_geojson_url + '?q=' + query);
        map.add_geojson_layer(function(geo_json) {
          var layers = geo_json.getLayers();
          var map_bounds = map.getBounds();
          var found = 0;
          var found_in_padded_bounds = false;
          if (layers.length == 0) return;
          for (var i=0; i<layers.length; i++) {
            if (map_bounds.contains(layers[i].getLatLng())) {
              found += 1;
            }
          }
          if (!found) {
            /* no visible marker */
            if (layers.length == 1) {
              map.flyTo(layers[0].getLatLng());
            }
            var bounds = geo_json.getBounds();
            map.flyToBounds(bounds);
          } else {
            if (map.getZoom() < 16) {
              if (layers.length == 1) {
                map.flyTo(layers[0].getLatLng(), 16);
              } else if (layers.length == found) {
                var bounds = geo_json.getBounds();
                map.flyToBounds(bounds);
              } else if (layers.length <= 10) {
                var bounds = geo_json.getBounds();
                map.flyToBounds(bounds);
              }
            } else {
              if (layers.length <= 10 && layers.length != found) {
                var bounds = geo_json.getBounds();
                map.flyToBounds(bounds);
              }
            }
          }
          $(document).trigger('gnm:new-results');
        });
        search_timeout_id = null;
      }, 300);
    });
    $(map).on('zoomend moveend', function() {
      $(document).trigger('gnm:new-results');
    });
    $(document).on('gnm:new-results', function() {
      if (map.clustered_markers) {
        map.clustered_markers.on('spiderfied', function() {
          $(document).trigger('gnm:new-results');
        });
      }
      if ($sidebar.length == 0) return;
      if ($sidebar.hasClass('loading')) return;
      $sidebar.empty();
      $sidebar.addClass('loading');
      var counter = 0;
      var tile_counter = 0;
      var cluster_groups = Array();
      map.eachLayer(function(layer) {
        if (layer instanceof L.Marker) {
          layer.matching_tile = null;
          if (! map.getBounds().contains(layer.getLatLng())) return;
          if (layer._childCount !== undefined) {
            cluster_groups.push(layer);
            return;
          }
          if (counter > 5) return;
          counter += 1;
          $.ajax({
            url: '/api/dashboard/auto-tile/' + layer.feature.properties.layer.identifier + '/',
            data: JSON.stringify(layer.feature.properties),
            contentType: 'application/json; charset=utf-8',
            type: 'POST',
            dataType: 'html',
            success: function(html) {
              var $tile = $('<div class="cell">' + html + '</div>');
              $sidebar.removeClass('empty');
              $tile.appendTo($sidebar);
              $tile[0].matching_marker = layer;
              layer.matching_tile = $tile;
              $tile.on('click', function() {
                map.flyTo(layer.getLatLng(), 15);
                return true;
              });
              tile_counter += 1;
              if (tile_counter == counter) {
                $sidebar.removeClass('loading');
              }
            }
          });
        }
      });
      if (counter == 0) {
        /* remove loading marker if there were no tiles */
        $sidebar.removeClass('loading');
      }
      if (counter == 0 && $('div.cell.map.loading').length == 0) {
        $sidebar.addClass('empty');
        var $empty = $('#empty-map-results');
        if ($empty.length == 0) {
          $empty = $('<p id="empty-map-results"></p>');
          $empty.text('Aucun résultat dans cette zone.');
          $empty.appendTo($sidebar);
        }
        if (cluster_groups.length) {
          $empty.text('Cliquez ou zoomez sur les clusters...');
        }
      }
    });
    if ($map_search_input.val()) { /* prefilled from query string */
      $map_search_input.trigger('change');
    } else {
      $('div.cell.map').on('combo:map-ready', function() {
        $(document).trigger('gnm:new-results');
      });
    }
  }

  /* add service count labels */
  if ($('.dashboardcell').length == 0) {
    $('.group-title').each(function() {
      var $group = $(this);
      var count = 0;
      var tile = $(this);
      while (true) {
        tile = $(tile).next();
        if (tile.length == 0) break;
        if (tile.hasClass('group-title')) {
          break;
        }
        if (! (tile.hasClass('tile') || tile.hasClass('linkcell'))) {
          break;
        }
        count += 1;
      }
      if (count == 1) {
        $(this).find('h2').after('<span>' + count + ' service</span>');
      } else if (count) {
        $(this).find('h2').after('<span>' + count + ' services</span>');
      }
    });
  }

  $('.dashboardcell .configjsoncell').each(function() {
    /* remove empty cells from DOM, this helps later operations such as
     * dashboard reordering to only have actual visible cells */
    if ($(this).height() == 0) {
      $(this).remove();
    }
  });

  /* section collapsing/expanding */
  $('div.group-title').on('click', function() {
    var expand = $(this).find('h2').hasClass('closed');
    var tile = $(this);
    var dashboard_parent = false;
    while (true) {
      tile = $(tile).next();
      if (tile.length == 0) break;
      if (tile.hasClass('dashboardcell')) {
        tile = $(tile).find('.cell');
        dashboard_parent = true;
      }
      if (tile.hasClass('wcscurrentformscell') || tile.hasClass('wcscurrentdraftscell')) {
        /* get inside */
        tile = $(tile).find('.tile');
      }
      if (tile.hasClass('group-title')) {
        break;
      }
      if (! (tile.hasClass('tile') || tile.hasClass('linkcell') || dashboard_parent)) {
        break;
      }
      if (expand) {
        $(tile).removeClass('closed');
      } else {
        $(tile).addClass('closed');
      }
    }
    if (expand) {
      $(this).find('h2').removeClass('closed');
    } else {
      $(this).find('h2').addClass('closed');
    }
  });

  $('div.dashboard-content div.cell').delegate('div.tile a', 'click', function(e) {
    e.stopPropagation();
  });

  /* dashboard ordering */
  $('div.dashboard-content div.cell').delegate('div.tile', 'click', function() {
    $(this).parents('.dashboard-content').toggleClass('edit-mode');
    return false;
  });

  $('div.dashboard-content div.cell').delegate('div.tile span.move-up', 'click', function() {
    if ($(this).parents('.edit-mode').length == 0) return;
    var $cell = $($(this).closest('.cell')[0]);
    var tile_bottom_margin = 14 / 2; /* see _tiles.css */
    var cell_height = $cell.height() + tile_bottom_margin;
    var $dashboard_content = $(this).closest('.dashboard-content');
    var $prev_cell = $($cell.prev()[0]);
    var prev_cell_height = $prev_cell.height() + tile_bottom_margin;
    $prev_cell.insertAfter($cell);
    var new_order = $.map($dashboard_content.find('> div'), function(val) { return $(val).data('tile-id'); }).join(',');
    $.ajax({
      url: $dashboard_content.data('dashboard-reorder-url') + '?order=' + new_order,
    });
    $cell.css('transition-duration', '0ms');
    $cell.css('z-index', '100');
    $prev_cell.css('transition-duration', '0ms');
    $cell[0].offsetHeight; /* commit */
    $cell.css('transform', 'translate(0px, ' + prev_cell_height + 'px)');
    $prev_cell.css('transform', 'translate(0px, -' + cell_height + 'px)');
    $cell[0].offsetHeight; /* commit */
    $cell.css('transition-duration', '500ms');
    $prev_cell.css('transition-duration', '500ms');
    $cell[0].offsetHeight; /* commit */
    $cell.css('transform', 'translate(0px, 0)');
    $prev_cell.css('transform', 'translate(0px, 0)');
    setTimeout(function() { $cell.css('z-index', '0'); }, 500);

    return false;
  });

  /* expandable tiles */
  $('div.cell').delegate('div.tile.expandable div.tile-content', 'click', function() {
    $(this).parents('.expandable').toggleClass('expanded');
    return false;
  });

  /* go to page */
  if ($('body').hasClass('section-index') ||
      $('body').hasClass('section-services') ||
      $('body').hasClass('section-bienvenue')) {
    /* on combo dashboard, search and welcome pages, always do geolocation so
     * it can be used with little delay on "go to" links */
    var geoloc_watch_id = null;

    navigator.geolocation.getCurrentPosition(
      function(position) {
        console.log('switching to watch'),
        geoloc_watch_id = navigator.geolocation.watchPosition(
          function(position) {
            console.log('watch got');
            $('body').addClass('has-geolocation');
            $('body').data('geoloc-lat', position.coords.latitude);
            $('body').data('geoloc-lng', position.coords.longitude);
          },
          function(error_watching) {
            console.log('error watching', error_watching);
          },
          {enableHighAccuracy: false, timeout: 5000, maximumAge: 5 * 60 * 1000}
        );
      },
      function(error_getting) {
        console.log('error getting', error_getting);
      },
      {enableHighAccuracy: false, timeout: Infinity, maximumAge: 20 * 60 * 1000}
    );
  }

  function getOnlyMoovDateString(){
    var now = new Date();
    var soon = new Date(now.getTime() + 60000);
    function pad(n) {return n<10 ? '0'+n : n}
    return soon.getFullYear() + '-' + pad(soon.getMonth()+1) +'-' + pad(soon.getDate()) + '_'
                  + pad(soon.getHours()) + '-' + pad(soon.getMinutes());
  }
  $('#main-content, div.cell').delegate('a.action-goto', 'click', function(e) {
    /* the delegation on div.cell is required for the dashboards, and the
     * delegtion on #main-content is required for search results. */
    var goto_url = '/m-y-rendre/?' + 'lat2=' + $(this).data('lat') + '&lng2=' + $(this).data('lng');
    var lat1 = $('body').data('geoloc-lat');
    var lng1 = $('body').data('geoloc-lng');
    var destname = $(this).closest('div.cell').find('.tile-title h2').text();
    if (lat1) {
      window.location = goto_url +
                      '&date=' + getOnlyMoovDateString() +
                      '&lat1=' + lat1 +
                      '&lng1=' + lng1 +
                      '&to=' + destname;
    } else {
      /* ask again for current position, with little chances of success given
       * the short timeout */
      navigator.geolocation.getCurrentPosition(
        function(pos) { /* success */
          window.location = goto_url +
                      '&date=' + getOnlyMoovDateString() +
                      '&lat1=' + pos.coords.latitude +
                      '&lng1=' + pos.coords.longitude +
                      '&to=' + destname;
        },
        function(err) { /* error */
          alert('Geolocation failed; is it enabled?');
        },
        {enableHighAccuracy: false, timeout: 5000, maximumAge: 1 * 60 * 1000}
      )
    }
    return false;
  });
  if ($('.section-m-y-rendre').length) {
    $('div.cell.map').on('combo:map-ready', function(ev) {
      var $map_widget = $(this).find('div.combo-cell-map');
      var map = $map_widget[0].leaflet_map;
      var marker1 = L.divIcon({iconAnchor: [0, 30],
              popupAnchor: [5, -45],
              html: '<span style="background: #00AA50;"><i class="leaflet-marker-icon start"></i></span>'});
      var marker2 = L.divIcon({iconAnchor: [0, 30],
              popupAnchor: [5, -45],
              html: '<span style="background: #D90000;"><i class="leaflet-marker-icon start"></i></span>'});

      var start = L.marker([parseFloat(getParameterByName('lat1')), parseFloat(getParameterByName('lng1'))],
              {'icon': marker1}).addTo(map);
      var finish = L.marker([parseFloat(getParameterByName('lat2')), parseFloat(getParameterByName('lng2'))],
              {'icon': marker2}).addTo(map);
      var group = new L.featureGroup([start, finish]);
      map.flyToBounds(group.getBounds());
    });
  }

  /* notifications */
  $('div.notification').on('click', function() {
    var cell = $(this);
    if ($(cell).hasClass('acked')) {
      if ($(cell).find('a').length) {
        window.location.href = $(cell).find('a').attr('href');
      }
      return;
    }
    var ack_url = $(this).data('combo-notification-ack-url');
    $.ajax({
         url: ack_url,
         success: function(html) {
           $(cell).removeClass('new').addClass('acked');
           if ($(cell).find('a').length) {
             window.location.href = $(cell).find('a').attr('href');
           }
         }
    });
  });

  /* similar demands */
  if ($('#map-similar').length) {
    var geojsonMarkerOptions = {
      radius: 8,
      fillColor: "#ff2000",
      color: "#000",
      weight: 1,
      opacity: 1,
      fillOpacity: 0.8
    };
    $(document).on('wcs:maps-ready', function() {
      $('.qommon-map').each(function() {
        var $map_widget = $(this);
        var map = $map_widget[0].leaflet_map;
        var geojson_url = $('#similar').data('geojson-url');
        var geo_json = null;

        $(map).on('zoomend moveend', function() {
          var similar_list = $('#similar-list');
          similar_list.find('> div').hide();
          map.eachLayer(function(layer){
            if (layer instanceof L.Marker && map.getBounds().contains(layer.getLatLng())) {
              if (layer.feature && layer.feature.properties.id) {
                similar_list.find('[data-id="' + layer.feature.properties.id + '"]').show();
                return;
              }
            }
          });
        });

        if (geojson_data) {
          var geo_json = L.geoJson(geojson_data,
            {
              pointToLayer: function (feature, latlng) {
                var markerStyles = "background-color: #292E48; font-size: 80%;";
                marker = L.divIcon({
                                    iconAnchor: [0, 0],
                                    popupAnchor: [5, -45],
                                    html: '<span style="' + markerStyles + '"><i class="leaflet-marker-icon ' +
                                        '" style="color: white; font-family: inherit; font-weight: bold;' +
                                        '">' + feature.properties.counter + '</i></span>'
                                   });
                return L.marker(latlng, {icon: marker});
              }
            });
          geo_json.addTo(map);
          $(map).trigger('moveend');
        }
      });
    });
  }

  /* collectivity selector */
  $('select#collectivities').on('change', function(a, b) {
    var $selected = $(this).find(':selected');
    if ($selected.data('url')) {
      window.location = $selected.data('url');
    }
  });

  $(document).on('combo:cell-loaded', function(ev, elem) {
    /* remove empty cells from DOM, this helps later operations such as
     * dashboard reordering to only have actual visible cells */
    if ($(elem).height() == 0) $(elem).remove();

    /* "your profile" cell */
    var drafts_count = $('.wcscurrentdraftscell .tile').length
    var forms_count = $('.wcscurrentformscell .tile').length
    if ($('.wcscurrentdraftscell .tile.info').length) drafts_count = 0;
    if ($('.wcscurrentformscell .tile.info').length) forms_count = 0;
    $('#drafts-count').text(drafts_count);
    $('#forms-count').text(forms_count);
    if (drafts_count < 2) {
      $('#drafts-count').next().text('brouillon');
    } else {
      $('#drafts-count').next().text('brouillons');
    }
    if (forms_count < 2) {
      $('#forms-count').next().text('demande');
    } else {
      $('#forms-count').next().text('demandes');
    }
  });

  /* reset error class when contents is changed */
  $('input, select, textarea').on('change', function() {
    $(this).parents('.widget-with-error').removeClass('widget-with-error');
  });

  /* service worker */
  if ('serviceWorker' in navigator) {
    navigator.serviceWorker.register('/service-worker.js', {scope: '/'}).then(function(registration) {
      // Registration was successful
      console.log('ServiceWorker registration successful with scope: ', registration.scope);
    }).catch(function(err) {
      // registration failed :(
      console.log('ServiceWorker registration failed: ', err);
    });
  }
});
