$(function() {
  var $sidebar = $('#sidebar');

  /* map search */
  $('.search-page div.cell.map').addClass('loading').on('combo:map-markers-ready', function() {
    $(this).removeClass('loading');
  });
  $('.search-page div.cell.map').on('combo:map-feature-prepare', function(ev, info) {
    info.layer.on('click', function(ev) {
      $('.leaflet-marker-icon.loading').removeClass('loading');
      if (info.layer.matching_tile) {
        if (info.layer.matching_tile.hasClass('selected')) {
          /* second click on same tile */
          var mql = window.matchMedia("(max-width: 1100px)");
          if (mql.matches) {
            $('#sidebar').css('min-height', 0);
            $('#sidebar .cell').removeClass('selected');
          }
          return;
        }
        $('#sidebar .cell').removeClass('selected');
        info.layer.matching_tile.addClass('selected');
        var $selected_cell = $('#sidebar .cell.selected');
        if ($selected_cell.prev().length) {
          $selected_cell.prependTo($selected_cell.parent());
        }
        /* on mobile, increase search zone height to make cell visible */
        var mql = window.matchMedia("(max-width: 1100px)");
        if (mql.matches) {
          $('#sidebar').css('min-height', $selected_cell.height() + 10);
        }
        return;
      }
      /* dynamically load tile and put it first */
      $(info.layer._icon).addClass('loading');
      $.ajax({
         url: '/api/dashboard/auto-tile/' + info.layer.feature.properties.layer.identifier + '/',
         data: JSON.stringify(ev.target.feature.properties),
         contentType: 'application/json; charset=utf-8',
         type: 'POST',
         dataType: 'html',
         success: function(html) {
           $(info.layer._icon).removeClass('loading');
           $('#sidebar .cell').removeClass('selected');
           var $tile = $('<div class="cell selected">' + html + '</div>');
           $tile.prependTo($sidebar);
           if ($tile.siblings().length > 1) {
             $tile.siblings().last()[0].matching_marker.matching_tile = null;
             $tile.siblings().last().remove();
           }
           $tile[0].matching_marker = info.layer;
           info.layer.matching_tile = $tile;

           /* on mobile, increase search zone height to make cell visible */
           var mql = window.matchMedia("(max-width: 1100px)");
           if (mql.matches) {
             $('#sidebar').css('min-height', $tile.height() + 10);
           }
         }
      });
    });
  });

  var $map_search_input = $('#mapsearch input');
  if ($map_search_input.length) {
    var $map_widget = $('div.combo-cell-map');
    var map = $map_widget[0].leaflet_map;
    var search_timeout_id = null;
    var base_geojson_url = $map_widget.data('geojson-url');
    $map_search_input.on('change keyup paste', function() {
      clearTimeout(search_timeout_id);
      search_timeout_id = setTimeout(function() {
        var query =  $('#mapsearch input').val();
        query = query.toLowerCase();
        query = query.replace(/^rue\s/i, '');
        if (query.length < 2) return;
        var geojson_url = base_geojson_url + '?q=' + query;
        if ($('body').hasClass('section-autour-de-moi')) {
          var center = map.getCenter();
          geojson_url += '&lat=' + center.lat + '&lng=' + center.lng + '&distance=1000';
        }

        $map_widget.data('geojson-url', geojson_url);
        map.add_geojson_layer(function(geo_json) {
          var layers = geo_json.getLayers();
          var map_bounds = map.getBounds();
          var found = 0;
          var found_in_padded_bounds = false;
          if (layers.length == 0) return;
          for (var i=0; i<layers.length; i++) {
            if (map_bounds.contains(layers[i].getLatLng())) {
              found += 1;
            }
          }
          if (!found) {
            /* no visible marker */
            if (layers.length == 1) {
              map.flyTo(layers[0].getLatLng());
            }
            var bounds = geo_json.getBounds();
            map.flyToBounds(bounds);
          } else {
            if (map.getZoom() < 16) {
              if (layers.length == 1) {
                map.flyTo(layers[0].getLatLng(), 16);
              } else if (layers.length == found) {
                var bounds = geo_json.getBounds();
                map.flyToBounds(bounds);
              } else if (layers.length <= 10) {
                var bounds = geo_json.getBounds();
                map.flyToBounds(bounds);
              }
            } else {
              if (layers.length <= 10 && layers.length != found) {
                var bounds = geo_json.getBounds();
                map.flyToBounds(bounds);
              }
            }
          }
          $(document).trigger('gnm:new-results');
        });
        search_timeout_id = null;
      }, 300);
    });
    $(map).on('movestart', function() {
      var mql = window.matchMedia("(max-width: 1100px)");
      if (mql.matches) {
        $('#sidebar').css('min-height', 0);
      }
    });
    $(map).on('zoomend moveend', function() {
      $(document).trigger('gnm:new-results');
    });
    $(document).on('gnm:new-results', function() {
      if (map.clustered_markers) {
        map.clustered_markers.on('spiderfied', function() {
          $(document).trigger('gnm:new-results');
        });
      }
      if ($sidebar.length == 0) return;
      if ($sidebar.hasClass('loading')) return;
      $sidebar.empty();
      $sidebar.addClass('loading');
      var counter = 0;
      var tile_counter = 0;
      var cluster_groups = Array();
      map.eachLayer(function(layer) {
        if (layer instanceof L.Marker) {
          layer.matching_tile = null;
          if (layer.gps_marker !== undefined) {
            return;
          }
          if (! map.getBounds().contains(layer.getLatLng())) return;
          if (layer._childCount !== undefined) {
            cluster_groups.push(layer);
            return;
          }
          if (counter > 5) return;
          counter += 1;
          $.ajax({
            url: '/api/dashboard/auto-tile/' + layer.feature.properties.layer.identifier + '/',
            data: JSON.stringify(layer.feature.properties),
            contentType: 'application/json; charset=utf-8',
            type: 'POST',
            dataType: 'html',
            success: function(html) {
              var $tile = $('<div class="cell">' + html + '</div>');
              $sidebar.removeClass('empty');
              $tile.appendTo($sidebar);
              $tile[0].matching_marker = layer;
              layer.matching_tile = $tile;
              $tile.on('click', function() {
                map.flyTo(layer.getLatLng(), 15);
                return true;
              });
              tile_counter += 1;
              if (tile_counter == counter) {
                $sidebar.removeClass('loading');
              }
            }
          });
        }
      });
      if (counter == 0) {
        /* remove loading marker if there were no tiles */
        $sidebar.removeClass('loading');
      }
      if (counter == 0 && $('div.cell.map.loading').length == 0) {
        $sidebar.addClass('empty');
        var $empty = $('#empty-map-results');
        if ($empty.length == 0) {
          $empty = $('<p id="empty-map-results"></p>');
          $empty.text('Aucun résultat dans cette zone.');
          $empty.appendTo($sidebar);
        }
        if (cluster_groups.length) {
          $empty.text('Cliquez ou zoomez sur les clusters...');
        }
      }
    });
    if ($map_search_input.val()) { /* prefilled from query string */
      $map_search_input.trigger('change');
    } else {
      $('div.cell.map').on('combo:map-ready', function() {
        $(document).trigger('gnm:new-results');
      });
    }
  }

  /* map on place pages */
  $('.is-place div.cell.map').on('combo:map-ready', function() {
    var $map_widget = $(this).find('div.combo-cell-map');
    var map = $map_widget[0].leaflet_map;
    var place_type = $(this).parents('.tile').data('place-type');
    if (place_type) {
      var marker = L.divIcon({
          iconAnchor: [0, 0],
          popupAnchor: [5, -45],
          html: '<span class="layer-' + place_type + '"><i class="leaflet-marker-icon"></i></span>'
      });
      L.marker([$map_widget.data('init-lat'), $map_widget.data('init-lng')], {icon: marker}).addTo(map);
    }
  });

  /* section collapsing/expanding */
  $('div.group-title').on('click', function() {
    var expand = $(this).hasClass('closed');
    var tile = $(this);
    var dashboard_parent = false;
    while (true) {
      tile = $(tile).next();
      if (tile.length == 0) break;
      if (tile.hasClass('dashboardcell')) {
        tile = $(tile).find('.cell');
        dashboard_parent = true;
      }
      if (tile.hasClass('wcscurrentformscell') || tile.hasClass('wcscurrentdraftscell')) {
        /* get inside */
        tile = $(tile).find('.tile');
      }
      if (tile.hasClass('group-title')) {
        break;
      }
      if (! (tile.hasClass('tile') || tile.hasClass('linkcell') || dashboard_parent)) {
        break;
      }
      if (expand) {
        $(tile).removeClass('closed');
      } else {
        $(tile).addClass('closed');
      }
    }
    if (expand) {
      $(this).removeClass('closed');
    } else {
      $(this).addClass('closed');
    }
  });

  /* add service count labels */
  if ($('.dashboardcell').length == 0) {
    $('.group-title').each(function() {
      var $group = $(this);
      var count = 0;
      var tile = $(this);
      while (true) {
        tile = $(tile).next();
        if (tile.length == 0) break;
        if (tile.hasClass('group-title')) {
          break;
        }
        if (! (tile.hasClass('tile') || tile.hasClass('linkcell'))) {
          break;
        }
        count += 1;
      }
      if (count) {
        $(this).find('h2').append(' <span class="count">(' + count + ')</span>');
      }
    });
  }

  /* collectivity selector */
  $('select#collectivities').on('change', function(a, b) {
    var $selected = $(this).find(':selected');
    if ($selected.data('url')) {
      window.location = $selected.data('url');
    }
  });

  /* notifications */
  $('div.notification').on('click', function() {
    var cell = $(this);
    if ($(cell).hasClass('acked')) {
      if ($(cell).find('a').length) {
        window.location.href = $(cell).find('a').attr('href');
      }
      return;
    }
    var ack_url = $(this).data('combo-notification-ack-url');
    $.ajax({
         url: ack_url,
         success: function(html) {
           $(cell).removeClass('new').addClass('acked');
           if ($(cell).find('a').length) {
             window.location.href = $(cell).find('a').attr('href');
           }
         }
    });
  });
  $('div.notification a.delete').on('click', function(event) {
    var notification = $(this).parents('.notification');
    var forget_url = $(notification).data('combo-notification-forget-url');
    $.ajax({
         url: forget_url,
         success: function(html) {
           $(notification).hide('fade');
         }
    });
    return false;
  });

  /* click on "connectez-vous" blocks */
  $('div#connectez-vous').on('click', function() {
    window.location = '/login/';
    return false;
  });

  $(document).on('combo:cell-loaded', function(ev, elem) {
    /* remove empty cells from DOM, this helps later operations such as
     * dashboard reordering to only have actual visible cells */
    if ($(elem).height() == 0) $(elem).remove();

    /* "your profile" cell */
    var drafts_count = $('.wcscurrentdraftscell .tile').length
    var forms_count = $('.wcscurrentformscell .tile').length
    if ($('.wcscurrentdraftscell .tile.info').length) drafts_count = 0;
    if ($('.wcscurrentformscell .tile.info').length) forms_count = 0;
    $('#drafts-count').text(drafts_count);
    $('#forms-count').text(forms_count);
    if (drafts_count < 2) {
      $('#drafts-count').next().text('brouillon');
    } else {
      $('#drafts-count').next().text('brouillons');
    }
    if (forms_count < 2) {
      $('#forms-count').next().text('demande');
    } else {
      $('#forms-count').next().text('demandes');
    }
  });



  /* service worker */
  if ('serviceWorker' in navigator) {
    navigator.serviceWorker.register('/service-worker.js', {scope: '/'}).then(function(registration) {
      // Registration was successful
      console.log('ServiceWorker registration successful with scope: ', registration.scope);
    }).catch(function(err) {
      // registration failed :(
      console.log('ServiceWorker registration failed: ', err);
    });
  }
});
